@@ -1,6 +1,6 @@
 package MyPageKit::Common;
 
-# $Id: Common.pm,v 1.8 2001/05/16 02:56:15 tjmather Exp $
+# $Id: Common.pm,v 1.19 2002/01/08 16:45:12 borisz Exp $
 
 use strict;
 
@@ -9,11 +9,14 @@ use vars qw(@ISA);
 
 use Apache::Constants qw(OK REDIRECT DECLINED);
 
-$__PACKAGE__::secret_md5 = 'you_should_place_your_own_md5_string_here';
-
+use Digest::MD5 ();
 use DBI;
+
+no strict 'refs';
+${ __PACKAGE__ . '::secret_md5' } = 'you_should_place_your_own_md5_string_here';
+use strict 'refs';
+
 use MyPageKit::MyModel;
-use Digest::MD5;
 
 sub pkit_dbi_connect {
   # this line should be replaced with a DBI->connect(...) statement
@@ -44,7 +47,12 @@ sub pkit_common_code {
   my $model = shift;
 
   # put code that is common to all pages here
-  my $session = $model->session;
+  my $session = {};
+
+  # only create new session if pkit_admin is set
+  if ($model->pkit_get_session_id || $model->input('pkit_admin')) {
+    $session = $model->session;
+  }
 
   # for the pagekit.org website, we control the colors based on the
   # values the user selected, stored in the session.
@@ -52,6 +60,12 @@ sub pkit_common_code {
   $model->output(text_color => $session->{'text_color'} || '000000');
   $model->output(bgcolor => $session->{'bgcolor'} || 'dddddd');
   $model->output(mod_color => $session->{'mod_color'} || 'ffffff');
+
+  # toggle on-line editing tools
+  if($model->input('pkit_admin')) {
+    $session->{'pkit_admin'} = $model->input('pkit_admin') eq 'on' ? 1 : 0;
+  }
+  $model->output(pkit_admin => $session->{'pkit_admin'});
 }
 
 sub pkit_auth_credential {
@@ -60,8 +74,8 @@ sub pkit_auth_credential {
   my $login = $model->input('login');
   my $passwd = $model->input('passwd');
 
-  unless ($login ne "" && $passwd ne ""){
-    $model->pkit_message("You did not fill all of the fields.  Please try again.",
+  unless ( $login && $passwd ){
+    $model->pkit_gettext_message("You did not fill all of the fields.  Please try again.",
 		 is_error => 1);
     return;
   }
@@ -71,13 +85,15 @@ sub pkit_auth_credential {
 
   my ($user_id, $dbpasswd) = $dbh->selectrow_array($sql_str, {}, $login);
 
-  unless ($epasswd eq crypt($dbpasswd,$epasswd)){
-    $model->pkit_message("Your login/password is invalid. Please try again.",
+  unless ($user_id && $dbpasswd && $epasswd eq crypt($dbpasswd,$epasswd)){
+    $model->pkit_gettext_message("Your login/password is invalid. Please try again.",
 		is_error => 1);
     return;
   }
 
-  my $hash = Digest::MD5->md5_hex(join ':', $__PACKAGE__::secret_md5, $user_id, $epasswd);
+  no strict 'refs';
+  my $hash = Digest::MD5::md5_hex(join ':', ${ __PACKAGE__ . '::secret_md5' }, $user_id, $epasswd);
+  use strict 'refs';
 
   my $ses_key = {
 		 'user_id'   => $user_id,
@@ -100,7 +116,9 @@ sub pkit_auth_session_key {
 
   # create a new hash and verify that it matches the supplied hash
   # (prevents tampering with the cookie)
-  my $newhash = Digest::MD5->md5_hex(join ':', $__PACKAGE__::secret_md5, $user_id, crypt($epasswd,"pk"));
+  no strict 'refs';
+  my $newhash = Digest::MD5::md5_hex(join ':', ${ __PACKAGE__ . '::secret_md5' }, $user_id, crypt($epasswd,"pk"));
+  use strict 'refs';
 
   return unless $newhash eq $ses_key->{'hash'};
 
@@ -108,7 +126,9 @@ sub pkit_auth_session_key {
 
   $model->output('pkit_login',$login);
 
-  return $user_id;
+  # the second variable return is the session_id, which in this application
+  # is the same as the user_id
+  return ($user_id, $user_id);
 }
 
 1;
@@ -123,83 +143,22 @@ MyPageKit::Common - Model class containing code common across site.
 
 This class contains methods that are common across the site, such
 as authentication and session key generation.  This particular class
-is an example class that is used for the pagekit.org website.
+is an example class that is used for the old pagekit.org website.
 It is derived from Apache::PageKit::Model and a base class for
 the Model classes for the pagekit.org site.
-In general, the class hierarchy should look like:
-
-		+---------------------------------------+
-		| 	  Apache::PageKit::Model	|
-		| Model code that provides an interface	|
-		| to PageKit and is common to all sites |
-		+---------------------------------------+
-				     |
-		+---------------------------------------+
-		|	    MyPageKit::Common		|
-		| Model code that is particular to the  |
-		| site, but common across all pages     |
-		+---------------------------------------+
-			/	    |		\
-    +----------------------------+     +----------------------------+
-    | MyPageKit::YourClass1      |     | MyPageKit::YourClassN      |
-    | Model code that is for a   | ... | Model code that is for a   |
-    | group of pages on the site |     | group of pages on the site |
-    +----------------------------+     +----------------------------+
 
 It is a good starting point for building your own base class for your
 Model classes.
 
-=head1 METHODS
-
-These are the methods that your should implement in your own MyPageKit::Common
-class:
-
-=over 4
-
-=item pkit_dbi_connect
-
-Should return a L<DBI> database handler object (C<$dbh>).  This
-object can be accessed from the rest of the Model by the method
-C<$model-E<gt>dbh>.
-
-=item pkit_session_setup
-
-Should return a hash reference with three key/value pairs:
-
-  * session_store_class
-	The object store class that should be used for
-	L<Apache::PageKit::Session> session handling.
-  * session_lock_class
-	The lock manager class that should be used for
-	L<Apache::PageKit::Session> session handling.
-  * session_args
-	Reference to an hash containing options for the
-	C<session_lock_class> and C<session_store_class>.
-
-=item pkit_auth_credential
-
-Should verify the user-supplied credentials and return a session key.  The
-session key can be any string - often you'll use the user ID and
-a MD5 hash of a a secret key, user ID, password.
-
-=item pkit_auth_session_key
-
-Should verify the session key (previously generated by C<auth_credential>)
-and return the user ID.
-
-=item pkit_common_code
-
-Executes code that is common across your site.
-
-=back
-
-=head1 AUTHOR
+=head1 AUTHORS
 
 T.J. Mather (tjmather@anidea.com)
 
+Boris Zentner (boris@id9.de)
+
 =head1 COPYRIGHT
 
-Copyright (c) 2000, AnIdea, Corp.  All rights Reserved.  PageKit is a trademark
+Copyright (c) 2000, 2001, 2002 AnIdea, Corp.  All rights Reserved.  PageKit is a trademark
 of AnIdea Corp.
 
 =head1 LICENSE
