@@ -20,6 +20,8 @@ sub new {
   }
   $self->{pkit_pk}->{output_param_object} ||= Apache::PageKit::Param->new();
   $self->{pkit_pk}->{fillinform_object} ||= Apache::PageKit::Param->new();
+  $self->{pkit_pk}->{ignore_fillinform_fields} ||= [];
+
   return $self;
 }
 
@@ -168,6 +170,10 @@ sub pkit_validate_input {
   }
 }
 
+sub pkit_component_params_hashref {
+  return $_[0]->{pkit_pk}->{component_params_hashref};
+}
+
 sub pkit_input_hashref {
   my $model = shift;
   return $model->{pkit_input_hashref} if
@@ -196,9 +202,12 @@ sub pkit_message {
     if ($@) {
       die "Charset $input_charset or $default_output_charset not supported by Text::Iconv";
     }
-    $message = $converter->convert($message) || die "Can not convert page from $input_charset to $default_output_charset";;
+    $message = $converter->convert($message) || die "Can not convert page from $input_charset to $default_output_charset" if $message;
   }
 
+  my $default_error_str = $model->pkit_get_config_attr( GLOBAL => 'default_errorstr' ) || "#ff0000";
+  $message  =~ s/<(!--)?\s*PKIT_ERRORSTR\s*(?(1)--)>/$default_error_str/gi;
+
   my $array_ref = $model->output('pkit_messages') || [];
   push @$array_ref, {pkit_message  => $message,
 		    pkit_is_error  => $options->{'is_error'}};
@@ -249,6 +258,13 @@ sub fillinform {
   return shift->{pkit_pk}->{fillinform_object}->param(@_);
 }
 
+# undocumented
+sub ignore_fillinform_fields {
+  my $model = shift;
+  push @{$model->{pkit_pk}->{ignore_fillinform_fields}}, @_;
+}
+
+
 sub output {
   return shift->{pkit_pk}->{output_param_object}->param(@_);
 }
@@ -442,7 +458,7 @@ sub _change_params {
       } elsif ( $type eq 'ARRAY' ) {
         _change_array( $converter, $_ );
       } else {
-        $_ = $converter->convert($_) || die "Can not convert from default_input_charset to default_output_charset";
+        $_ = $converter->convert($_) || die "Can not convert from default_input_charset to default_output_charset" if $_;
       }
     }
   }
@@ -456,7 +472,7 @@ sub _change_params {
       } elsif ( $type eq 'ARRAY' ) {
         _change_array( $converter, $_ );
       } else {
-        $_ = $converter->convert($_) || die "Can not convert from default_input_charset to default_output_charset";
+        $_ = $converter->convert($_) || die "Can not convert from default_input_charset to default_output_charset" if $_;
       }
     }
   }
@@ -468,7 +484,7 @@ sub _change_params {
     } elsif ( $type eq 'ARRAY' ) {
       _change_array( $converter, $_[$i] );
     } else {
-      $_[$i] = $converter->convert($_[$i]) || die "Can not convert from default_input_charset to default_output_charset";
+      $_[$i] = $converter->convert($_[$i]) || die "Can not convert from default_input_charset to default_output_charset" if $_[$i];
     }
   }
 }
